/* 
 * irctrl-cl.h: client engine header
 *
 *	By:	Alan Yates <alany@ay.com.au>
 */

/* code table */
struct code_t {
	char *name;
	char *code;
};

#define BUFSIZE 4096	/* buffer size */
#define IR_DEV "irctrl"	/* device file */
extern int DEV;		/* device fd */
extern char *PROG;	/* argv[0] */
extern unsigned int recv_cursor;	/* receive buffer cursor */
extern char rxbuffer[];	/* receive buffer */

/* THE API */
//////
//
//	* send
//	int protocol_tx(char *cmd)
//	cmd is the name of packet to send
//
//	* receive
//	int protocol_rx(int *rxcursor)
//	gets passed the current place cursor, expected to modify it if it
//	decodes a frame from the buffer
//
//////
