#!/usr/bin/awk -f
###
# Script to print usec delay between struct timeval vals
###
{
	if(!new) {
		startsec = $1;
		startusec = $2;
		new = 1;
	}

	sectime = $1 - startsec;
	usectime = $2 - startusec

	time = sectime * 1000000 + usectime;
	delay = time - lasttime;

	printf("%ld\n", delay);

	lasttime = time;

}
